package java.awt.geom;

import java.io.Serializable;
import java.io.StreamCorruptedException;
import java.io.InvalidObjectException;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import sun.awt.geom.Curve;
import java.awt.Rectangle;
import java.util.Arrays;
import java.awt.Shape;

public abstract class Path2D implements Shape, Cloneable
{
    public static final int WIND_EVEN_ODD = 0;
    public static final int WIND_NON_ZERO = 1;
    private static final byte SEG_MOVETO = 0;
    private static final byte SEG_LINETO = 1;
    private static final byte SEG_QUADTO = 2;
    private static final byte SEG_CUBICTO = 3;
    private static final byte SEG_CLOSE = 4;
    transient byte[] pointTypes;
    transient int numTypes;
    transient int numCoords;
    transient int windingRule;
    static final int INIT_SIZE = 20;
    static final int EXPAND_MAX = 500;
    static final int EXPAND_MAX_COORDS = 1000;
    static final int EXPAND_MIN = 10;
    private static final byte SERIAL_STORAGE_FLT_ARRAY = 48;
    private static final byte SERIAL_STORAGE_DBL_ARRAY = 49;
    private static final byte SERIAL_SEG_FLT_MOVETO = 64;
    private static final byte SERIAL_SEG_FLT_LINETO = 65;
    private static final byte SERIAL_SEG_FLT_QUADTO = 66;
    private static final byte SERIAL_SEG_FLT_CUBICTO = 67;
    private static final byte SERIAL_SEG_DBL_MOVETO = 80;
    private static final byte SERIAL_SEG_DBL_LINETO = 81;
    private static final byte SERIAL_SEG_DBL_QUADTO = 82;
    private static final byte SERIAL_SEG_DBL_CUBICTO = 83;
    private static final byte SERIAL_SEG_CLOSE = 96;
    private static final byte SERIAL_PATH_END = 97;
    
    Path2D() {
    }
    
    Path2D(int windingRule, int n) {
        this.setWindingRule(windingRule);
        this.pointTypes = new byte[n];
    }
    
    abstract float[] cloneCoordsFloat(AffineTransform p0);
    
    abstract double[] cloneCoordsDouble(AffineTransform p0);
    
    abstract void append(float p0, float p1);
    
    abstract void append(double p0, double p1);
    
    abstract Point2D getPoint(int p0);
    
    abstract void needRoom(boolean p0, int p1);
    
    abstract int pointCrossings(double p0, double p1);
    
    abstract int rectCrossings(double p0, double p1, double p2, double p3);
    
    static byte[] expandPointTypes(byte[] array, int n) {
        int n2 = array.length;
        int n3 = n2 + n;
        if (n3 < n2) {
            throw new ArrayIndexOutOfBoundsException("pointTypes exceeds maximum capacity !");
        }
        if ((n2 = n2) > 500) {
            n2 = Math.max(500, n2 >> 3);
        }
        else if (n2 < 10) {
            n2 = 10;
        }
        assert n2 > 0;
        int n4 = n2 + n2;
        if (n4 < n3) {
            n4 = Integer.MAX_VALUE;
        }
        try {
            return Arrays.copyOf(array, n4);
        }
        catch (OutOfMemoryError outOfMemoryError) {
            if (n4 == n3) {
                throw outOfMemoryError;
            }
            n4 = n3 + (n4 - n3) / 2;
            return Arrays.copyOf(array, n4);
        }
    }
    
    public abstract void moveTo(double p0, double p1);
    
    public abstract void lineTo(double p0, double p1);
    
    public abstract void quadTo(double p0, double p1, double p2, double p3);
    
    public abstract void curveTo(double p0, double p1, double p2, double p3, double p4, double p5);
    
    public final synchronized void closePath() {
        if (this.numTypes == 0 || this.pointTypes[this.numTypes - 1] != 4) {
            this.needRoom(true, 0);
            this.pointTypes[this.numTypes++] = 4;
        }
    }
    
    public final void append(Shape shape, boolean b) {
        this.append(shape.getPathIterator(null), b);
    }
    
    public abstract void append(PathIterator p0, boolean p1);
    
    public final synchronized int getWindingRule() {
        return this.windingRule;
    }
    
    public final void setWindingRule(int windingRule) {
        if (windingRule != 0 && windingRule != 1) {
            throw new IllegalArgumentException("winding rule must be WIND_EVEN_ODD or WIND_NON_ZERO");
        }
        this.windingRule = windingRule;
        return;
    }
    
    public final synchronized Point2D getCurrentPoint() {
        int numCoords = this.numCoords;
        if (this.numTypes < 1 || numCoords < 1) {
            return null;
        }
        Label_0115: {
            if (this.pointTypes[this.numTypes - 1] == 4) {
                for (int i = this.numTypes - 2; i > 0; --i) {
                    switch (this.pointTypes[i]) {
                        case 0: {
                            break Label_0115;
                        }
                        case 1: {
                            numCoords -= 2;
                            break;
                        }
                        case 2: {
                            numCoords -= 4;
                            break;
                        }
                        case 3: {
                            numCoords -= 6;
                            break;
                        }
                    }
                }
            }
        }
        return this.getPoint(numCoords - 2);
    }
    
    public final synchronized void reset() {
        int n = 0;
        this.numCoords = n;
        this.numTypes = n;
    }
    
    public abstract void transform(AffineTransform p0);
    
    public final synchronized Shape createTransformedShape(AffineTransform affineTransform) {
        Path2D path2D = (Path2D)this.clone();
        if (affineTransform != null) {
            path2D.transform(affineTransform);
            return path2D;
        }
        return path2D;
    }
    
    @Override
    public final Rectangle getBounds() {
        return this.getBounds2D().getBounds();
    }
    
    public static boolean contains(PathIterator pi, double px, double py) {
        return px * 0.0 + py * 0.0 == 0.0 && (Curve.pointCrossingsForPath(pi, px, py) & ((pi.getWindingRule() == 1) ? -1 : 1)) != 0x0;
    }
    
    public static boolean contains(PathIterator pathIterator, Point2D point2D) {
        return contains(pathIterator, point2D.getX(), point2D.getY());
    }
    
    @Override
    public final boolean contains(double n, double n2) {
        return n * 0.0 + n2 * 0.0 == 0.0 && this.numTypes >= 2 && (this.pointCrossings(n, n2) & ((this.windingRule == 1) ? -1 : 1)) != 0x0;
    }
    
    @Override
    public final boolean contains(Point2D point2D) {
        return this.contains(point2D.getX(), point2D.getY());
    }
    
    public static boolean contains(PathIterator pi, double rxmin, double rymin, double n, double n2) {
        if (java.lang.Double.isNaN(rxmin + n) || java.lang.Double.isNaN(rymin + n2)) {
            return false;
        }
        if (n <= 0.0 || n2 <= 0.0) {
            return false;
        }
        int n3 = (pi.getWindingRule() == 1) ? -1 : 2;
        int rectCrossingsForPath = Curve.rectCrossingsForPath(pi, rxmin, rymin, rxmin + n, rymin + n2);
        return rectCrossingsForPath != Integer.MIN_VALUE && (rectCrossingsForPath & n3) != 0x0;
    }
    
    public static boolean contains(PathIterator pathIterator, Rectangle2D rectangle2D) {
        return contains(pathIterator, rectangle2D.getX(), rectangle2D.getY(), rectangle2D.getWidth(), rectangle2D.getHeight());
    }
    
    @Override
    public final boolean contains(double n, double n2, double n3, double n4) {
        if (java.lang.Double.isNaN(n + n3) || java.lang.Double.isNaN(n2 + n4)) {
            return false;
        }
        if (n3 <= 0.0 || n4 <= 0.0) {
            return false;
        }
        int n5 = (this.windingRule == 1) ? -1 : 2;
        int rectCrossings = this.rectCrossings(n, n2, n + n3, n2 + n4);
        return rectCrossings != Integer.MIN_VALUE && (rectCrossings & n5) != 0x0;
    }
    
    @Override
    public final boolean contains(Rectangle2D rectangle2D) {
        return this.contains(rectangle2D.getX(), rectangle2D.getY(), rectangle2D.getWidth(), rectangle2D.getHeight());
    }
    
    public static boolean intersects(PathIterator pi, double rxmin, double rymin, double n, double n2) {
        if (java.lang.Double.isNaN(rxmin + n) || java.lang.Double.isNaN(rymin + n2)) {
            return false;
        }
        if (n <= 0.0 || n2 <= 0.0) {
            return false;
        }
        int n3 = (pi.getWindingRule() == 1) ? -1 : 2;
        int rectCrossingsForPath = Curve.rectCrossingsForPath(pi, rxmin, rymin, rxmin + n, rymin + n2);
        return rectCrossingsForPath == Integer.MIN_VALUE || (rectCrossingsForPath & n3) != 0x0;
    }
    
    public static boolean intersects(PathIterator pathIterator, Rectangle2D rectangle2D) {
        return intersects(pathIterator, rectangle2D.getX(), rectangle2D.getY(), rectangle2D.getWidth(), rectangle2D.getHeight());
    }
    
    @Override
    public final boolean intersects(double n, double n2, double n3, double n4) {
        if (java.lang.Double.isNaN(n + n3) || java.lang.Double.isNaN(n2 + n4)) {
            return false;
        }
        if (n3 <= 0.0 || n4 <= 0.0) {
            return false;
        }
        int n5 = (this.windingRule == 1) ? -1 : 2;
        int rectCrossings = this.rectCrossings(n, n2, n + n3, n2 + n4);
        return rectCrossings == Integer.MIN_VALUE || (rectCrossings & n5) != 0x0;
    }
    
    @Override
    public final boolean intersects(Rectangle2D rectangle2D) {
        return this.intersects(rectangle2D.getX(), rectangle2D.getY(), rectangle2D.getWidth(), rectangle2D.getHeight());
    }
    
    @Override
    public final PathIterator getPathIterator(AffineTransform affineTransform, double n) {
        return new FlatteningPathIterator(this.getPathIterator(affineTransform), n);
    }
    
    public abstract Object clone();
    
    final void writeObject(ObjectOutputStream objectOutputStream, boolean b) throws IOException {
        objectOutputStream.defaultWriteObject();
        float[] floatCoords;
        double[] doubleCoords;
        if (b) {
            doubleCoords = ((Path2D.Double)this).doubleCoords;
            floatCoords = null;
        }
        else {
            floatCoords = ((Path2D.Float)this).floatCoords;
            doubleCoords = null;
        }
        int numTypes = this.numTypes;
        objectOutputStream.writeByte(b ? 49 : 48);
        objectOutputStream.writeInt(numTypes);
        objectOutputStream.writeInt(this.numCoords);
        objectOutputStream.writeByte((int)(byte)this.windingRule);
        int n = 0;
        for (int i = 0; i < numTypes; ++i) {
            int n2;
            int val;
            switch (this.pointTypes[i]) {
                case 0: {
                    n2 = 1;
                    val = (b ? 80 : 64);
                    break;
                }
                case 1: {
                    n2 = 1;
                    val = (b ? 81 : 65);
                    break;
                }
                case 2: {
                    n2 = 2;
                    val = (b ? 82 : 66);
                    break;
                }
                case 3: {
                    n2 = 3;
                    val = (b ? 83 : 67);
                    break;
                }
                case 4: {
                    n2 = 0;
                    val = 96;
                    break;
                }
                default: {
                    throw new InternalError("unrecognized path type");
                }
            }
            objectOutputStream.writeByte(val);
            while (--n2 >= 0) {
                if (b) {
                    objectOutputStream.writeDouble(doubleCoords[n++]);
                    objectOutputStream.writeDouble(doubleCoords[n++]);
                }
                else {
                    objectOutputStream.writeFloat(floatCoords[n++]);
                    objectOutputStream.writeFloat(floatCoords[n++]);
                }
            }
        }
        objectOutputStream.writeByte(97);
    }
    
    final void readObject(ObjectInputStream objectInputStream, boolean b) throws ClassNotFoundException, IOException {
        objectInputStream.defaultReadObject();
        objectInputStream.readByte();
        int int1 = objectInputStream.readInt();
        int int2 = objectInputStream.readInt();
        try {
            this.setWindingRule((int)objectInputStream.readByte());
        }
        catch (IllegalArgumentException ex) {
            throw new InvalidObjectException(ex.getMessage());
        }
        this.pointTypes = new byte[(int1 < 0 || int1 > 20) ? 20 : int1];
        if (int2 < 0 || int2 > 40) {
            int2 = 40;
        }
        if (b) {
            ((Path2D.Double)this).doubleCoords = new double[int2];
        }
        else {
            ((Path2D.Float)this).floatCoords = new float[int2];
        }
        int n4;
    Label_0512:
        for (int n = 0; int1 < 0 || n < int1; this.pointTypes[this.numTypes++] = n4, ++n) {
            int n2;
            int n3;
            switch (objectInputStream.readByte()) {
                case 64: {
                    n2 = false;
                    n3 = 1;
                    n4 = 0;
                    break;
                }
                case 65: {
                    n2 = false;
                    n3 = 1;
                    n4 = 1;
                    break;
                }
                case 66: {
                    n2 = false;
                    n3 = 2;
                    n4 = 2;
                    break;
                }
                case 67: {
                    n2 = false;
                    n3 = 3;
                    n4 = 3;
                    break;
                }
                case 80: {
                    n2 = true;
                    n3 = 1;
                    n4 = 0;
                    break;
                }
                case 81: {
                    n2 = true;
                    n3 = 1;
                    n4 = 1;
                    break;
                }
                case 82: {
                    n2 = true;
                    n3 = 2;
                    n4 = 2;
                    break;
                }
                case 83: {
                    n2 = true;
                    n3 = 3;
                    n4 = 3;
                    break;
                }
                case 96: {
                    n2 = false;
                    n3 = 0;
                    n4 = 4;
                    break;
                }
                case 97: {
                    if (int1 < 0) {
                        break Label_0512;
                    }
                    throw new StreamCorruptedException("unexpected PATH_END");
                }
                default: {
                    throw new StreamCorruptedException("unrecognized path type");
                }
            }
            this.needRoom(n4 != 0, n3 * 2);
            if (!(n2 == 0)) {
                while (--n3 >= 0) {
                    this.append(objectInputStream.readDouble(), objectInputStream.readDouble());
                }
            }
            else {
                while (--n3 >= 0) {
                    this.append(objectInputStream.readFloat(), objectInputStream.readFloat());
                }
            }
        }
        if (int1 >= 0 && objectInputStream.readByte() != 97) {
            throw new StreamCorruptedException("missing PATH_END");
        }
        return;
    }
    
    public static class Float extends Path2D implements Serializable
    {
        transient float[] floatCoords;
        private static final long serialVersionUID = 6990832515060788886L;
        
        public Float() {
            this(1, 20);
        }
        
        public Float(int n) {
            this(n, 20);
        }
        
        public Float(int n, int n2) {
            super(n, n2);
            this.floatCoords = new float[n2 * 2];
        }
        
        public Float(Shape shape) {
            this(shape, null);
        }
        
        public Float(Shape shape, AffineTransform affineTransform) {
            if ((shape instanceof Path2D)) {
                Path2D path2D = (Path2D)shape;
                this.setWindingRule(path2D.windingRule);
                this.numTypes = path2D.numTypes;
                this.pointTypes = Arrays.copyOf(path2D.pointTypes, path2D.numTypes);
                this.numCoords = path2D.numCoords;
                this.floatCoords = path2D.cloneCoordsFloat(affineTransform);
            }
            else {
                PathIterator pathIterator = shape.getPathIterator(affineTransform);
                this.setWindingRule(pathIterator.getWindingRule());
                this.pointTypes = new byte[20];
                this.floatCoords = new float[40];
                this.append(pathIterator, false);
            }
        }
        
        @Override
        float[] cloneCoordsFloat(AffineTransform affineTransform) {
            if (affineTransform == null) {
                return Arrays.copyOf(this.floatCoords, this.numCoords);
            }
            float[] copy = new float[this.numCoords];
            affineTransform.transform(this.floatCoords, 0, copy, 0, this.numCoords / 2);
            return copy;
        }
        
        @Override
        double[] cloneCoordsDouble(AffineTransform affineTransform) {
            double[] array = new double[this.numCoords];
            if (affineTransform == null) {
                for (int i = 0; i < this.numCoords; ++i) {
                    array[i] = (double)this.floatCoords[i];
                }
                return array;
            }
            affineTransform.transform(this.floatCoords, 0, array, 0, this.numCoords / 2);
            return array;
        }
        
        @Override
        void append(float n, float n2) {
            this.floatCoords[this.numCoords++] = n;
            this.floatCoords[this.numCoords++] = n2;
        }
        
        @Override
        void append(double n, double n2) {
            this.floatCoords[this.numCoords++] = (float)n;
            this.floatCoords[this.numCoords++] = (float)n2;
        }
        
        @Override
        Point2D getPoint(int n) {
            return new Point2D.Float(this.floatCoords[n], this.floatCoords[n + 1]);
        }
        
        @Override
        void needRoom(boolean b, int n) {
            if (this.numTypes == 0 && b) {
                throw new IllegalPathStateException("missing initial moveto in path definition");
            }
            if (this.numTypes >= this.pointTypes.length) {
                this.pointTypes = Path2D.expandPointTypes(this.pointTypes, 1);
            }
            if (this.numCoords > this.floatCoords.length - n) {
                this.floatCoords = expandCoords(this.floatCoords, n);
            }
            return;
        }
        
        static float[] expandCoords(float[] array, int n) {
            int n2 = array.length;
            int n3 = n2 + n;
            if (n3 < n2) {
                throw new ArrayIndexOutOfBoundsException("coords exceeds maximum capacity !");
            }
            if ((n2 = n2) > 1000) {
                n2 = Math.max(1000, n2 >> 3);
            }
            else if (n2 < 10) {
                n2 = 10;
            }
            assert n2 > n;
            int n4 = n2 + n2;
            if (n4 < n3) {
                n4 = Integer.MAX_VALUE;
            }
            try {
                return Arrays.copyOf(array, n4);
            }
            catch (OutOfMemoryError outOfMemoryError) {
                if (n4 == n3) {
                    throw outOfMemoryError;
                }
                n4 = n3 + (n4 - n3) / 2;
                return Arrays.copyOf(array, n4);
            }
        }
        
        @Override
        public final synchronized void moveTo(double n, double n2) {
            if (this.numTypes > 0 && this.pointTypes[this.numTypes - 1] == 0) {
                this.floatCoords[this.numCoords - 2] = (float)n;
                this.floatCoords[this.numCoords - 1] = (float)n2;
            }
            else {
                this.needRoom(false, 2);
                this.pointTypes[this.numTypes++] = 0;
                this.floatCoords[this.numCoords++] = (float)n;
                this.floatCoords[this.numCoords++] = (float)n2;
            }
        }
        
        public final synchronized void moveTo(float n, float n2) {
            if (this.numTypes > 0 && this.pointTypes[this.numTypes - 1] == 0) {
                this.floatCoords[this.numCoords - 2] = n;
                this.floatCoords[this.numCoords - 1] = n2;
            }
            else {
                this.needRoom(false, 2);
                this.pointTypes[this.numTypes++] = 0;
                this.floatCoords[this.numCoords++] = n;
                this.floatCoords[this.numCoords++] = n2;
            }
        }
        
        @Override
        public final synchronized void lineTo(double n, double n2) {
            this.needRoom(true, 2);
            this.pointTypes[this.numTypes++] = 1;
            this.floatCoords[this.numCoords++] = (float)n;
            this.floatCoords[this.numCoords++] = (float)n2;
        }
        
        public final synchronized void lineTo(float n, float n2) {
            this.needRoom(true, 2);
            this.pointTypes[this.numTypes++] = 1;
            this.floatCoords[this.numCoords++] = n;
            this.floatCoords[this.numCoords++] = n2;
        }
        
        @Override
        public final synchronized void quadTo(double n, double n2, double n3, double n4) {
            this.needRoom(true, 4);
            this.pointTypes[this.numTypes++] = 2;
            this.floatCoords[this.numCoords++] = (float)n;
            this.floatCoords[this.numCoords++] = (float)n2;
            this.floatCoords[this.numCoords++] = (float)n3;
            this.floatCoords[this.numCoords++] = (float)n4;
        }
        
        public final synchronized void quadTo(float n, float n2, float n3, float n4) {
            this.needRoom(true, 4);
            this.pointTypes[this.numTypes++] = 2;
            this.floatCoords[this.numCoords++] = n;
            this.floatCoords[this.numCoords++] = n2;
            this.floatCoords[this.numCoords++] = n3;
            this.floatCoords[this.numCoords++] = n4;
        }
        
        @Override
        public final synchronized void curveTo(double n, double n2, double n3, double n4, double n5, double n6) {
            this.needRoom(true, 6);
            this.pointTypes[this.numTypes++] = 3;
            this.floatCoords[this.numCoords++] = (float)n;
            this.floatCoords[this.numCoords++] = (float)n2;
            this.floatCoords[this.numCoords++] = (float)n3;
            this.floatCoords[this.numCoords++] = (float)n4;
            this.floatCoords[this.numCoords++] = (float)n5;
            this.floatCoords[this.numCoords++] = (float)n6;
        }
        
        public final synchronized void curveTo(float n, float n2, float n3, float n4, float n5, float n6) {
            this.needRoom(true, 6);
            this.pointTypes[this.numTypes++] = 3;
            this.floatCoords[this.numCoords++] = n;
            this.floatCoords[this.numCoords++] = n2;
            this.floatCoords[this.numCoords++] = n3;
            this.floatCoords[this.numCoords++] = n4;
            this.floatCoords[this.numCoords++] = n5;
            this.floatCoords[this.numCoords++] = n6;
        }
        
        @Override
        int pointCrossings(double n, double n2) {
            if (this.numTypes == 0) {
                return 0;
            }
            float[] floatCoords = this.floatCoords;
            double x2;
            double x1 = x2 = (double)floatCoords[0];
            double y2;
            double y1 = y2 = (double)floatCoords[1];
            int n3 = 0;
            int n4 = 2;
            for (int i = 1; i < this.numTypes; ++i) {
                switch (this.pointTypes[i]) {
                    case 0: {
                        if (y2 != y1) {
                            n3 += Curve.pointCrossingsForLine(n, n2, x2, y2, x1, y1);
                        }
                        x2 = (x1 = (double)floatCoords[n4++]);
                        y2 = (y1 = (double)floatCoords[n4++]);
                        break;
                    }
                    case 1: {
                        int n5 = n3;
                        double x3 = x2;
                        double y3 = y2;
                        double x4 = (double)floatCoords[n4++];
                        double n6;
                        n3 = n5 + Curve.pointCrossingsForLine(n, n2, x3, y3, x4, n6 = (double)floatCoords[n4++]);
                        x2 = x4;
                        y2 = n6;
                        break;
                    }
                    case 2: {
                        int n7 = n3;
                        double x5 = x2;
                        double y4 = y2;
                        double xc = (double)floatCoords[n4++];
                        double yc = (double)floatCoords[n4++];
                        double x6 = (double)floatCoords[n4++];
                        double n8;
                        n3 = n7 + Curve.pointCrossingsForQuad(n, n2, x5, y4, xc, yc, x6, n8 = (double)floatCoords[n4++], 0);
                        x2 = x6;
                        y2 = n8;
                        break;
                    }
                    case 3: {
                        int n9 = n3;
                        double x7 = x2;
                        double y5 = y2;
                        double xc2 = (double)floatCoords[n4++];
                        double yc2 = (double)floatCoords[n4++];
                        double xc3 = (double)floatCoords[n4++];
                        double yc3 = (double)floatCoords[n4++];
                        double x8 = (double)floatCoords[n4++];
                        double n10;
                        n3 = n9 + Curve.pointCrossingsForCubic(n, n2, x7, y5, xc2, yc2, xc3, yc3, x8, n10 = (double)floatCoords[n4++], 0);
                        x2 = x8;
                        y2 = n10;
                        break;
                    }
                    case 4: {
                        if (y2 != y1) {
                            n3 += Curve.pointCrossingsForLine(n, n2, x2, y2, x1, y1);
                        }
                        x2 = x1;
                        y2 = y1;
                        break;
                    }
                }
            }
            if (y2 != y1) {
                n3 += Curve.pointCrossingsForLine(n, n2, x2, y2, x1, y1);
            }
            return n3;
        }
        
        @Override
        int rectCrossings(double n, double n2, double n3, double n4) {
            if (this.numTypes == 0) {
                return 0;
            }
            float[] floatCoords = this.floatCoords;
            double x2;
            double x1 = x2 = (double)floatCoords[0];
            double y2;
            double y1 = y2 = (double)floatCoords[1];
            int crossings = 0;
            int n5 = 2;
            for (int n6 = 1; crossings != Integer.MIN_VALUE && n6 < this.numTypes; ++n6) {
                switch (this.pointTypes[n6]) {
                    case 0: {
                        if (x2 != x1 || y2 != y1) {
                            crossings = Curve.rectCrossingsForLine(crossings, n, n2, n3, n4, x2, y2, x1, y1);
                        }
                        x2 = (x1 = (double)floatCoords[n5++]);
                        y2 = (y1 = (double)floatCoords[n5++]);
                        break;
                    }
                    case 1: {
                        int crossings2 = crossings;
                        double x3 = x2;
                        double y3 = y2;
                        double x4 = (double)floatCoords[n5++];
                        double n7;
                        crossings = Curve.rectCrossingsForLine(crossings2, n, n2, n3, n4, x3, y3, x4, n7 = (double)floatCoords[n5++]);
                        x2 = x4;
                        y2 = n7;
                        break;
                    }
                    case 2: {
                        int crossings3 = crossings;
                        double x5 = x2;
                        double y4 = y2;
                        double xc = (double)floatCoords[n5++];
                        double yc = (double)floatCoords[n5++];
                        double x6 = (double)floatCoords[n5++];
                        double n8;
                        crossings = Curve.rectCrossingsForQuad(crossings3, n, n2, n3, n4, x5, y4, xc, yc, x6, n8 = (double)floatCoords[n5++], 0);
                        x2 = x6;
                        y2 = n8;
                        break;
                    }
                    case 3: {
                        int crossings4 = crossings;
                        double x7 = x2;
                        double y5 = y2;
                        double xc2 = (double)floatCoords[n5++];
                        double yc2 = (double)floatCoords[n5++];
                        double xc3 = (double)floatCoords[n5++];
                        double yc3 = (double)floatCoords[n5++];
                        double x8 = (double)floatCoords[n5++];
                        double n9;
                        crossings = Curve.rectCrossingsForCubic(crossings4, n, n2, n3, n4, x7, y5, xc2, yc2, xc3, yc3, x8, n9 = (double)floatCoords[n5++], 0);
                        x2 = x8;
                        y2 = n9;
                        break;
                    }
                    case 4: {
                        if (x2 != x1 || y2 != y1) {
                            crossings = Curve.rectCrossingsForLine(crossings, n, n2, n3, n4, x2, y2, x1, y1);
                        }
                        x2 = x1;
                        y2 = y1;
                        break;
                    }
                }
            }
            if (crossings != Integer.MIN_VALUE && (x2 != x1 || y2 != y1)) {
                crossings = Curve.rectCrossingsForLine(crossings, n, n2, n3, n4, x2, y2, x1, y1);
            }
            return crossings;
        }
        
        @Override
        public final void append(PathIterator pathIterator, boolean b) {
            float[] array = new float[6];
            while (!pathIterator.isDone()) {
                switch (pathIterator.currentSegment(array)) {
                    case 0: {
                        if (!b || this.numTypes < 1 || this.numCoords < 1) {
                            this.moveTo(array[0], array[1]);
                            break;
                        }
                        if (this.pointTypes[this.numTypes - 1] != 4 && this.floatCoords[this.numCoords - 2] == array[0] && this.floatCoords[this.numCoords - 1] == array[1]) {
                            break;
                        }
                        this.lineTo(array[0], array[1]);
                        break;
                    }
                    case 1: {
                        this.lineTo(array[0], array[1]);
                        break;
                    }
                    case 2: {
                        this.quadTo(array[0], array[1], array[2], array[3]);
                        break;
                    }
                    case 3: {
                        this.curveTo(array[0], array[1], array[2], array[3], array[4], array[5]);
                        break;
                    }
                    case 4: {
                        this.closePath();
                        break;
                    }
                }
                pathIterator.next();
                b = false;
            }
        }
        
        @Override
        public final void transform(AffineTransform affineTransform) {
            affineTransform.transform(this.floatCoords, 0, this.floatCoords, 0, this.numCoords / 2);
        }
        
        @Override
        public final synchronized Rectangle2D getBounds2D() {
            int i = this.numCoords;
            float n4;
            float n3;
            float n2;
            float n;
            if (i > 0) {
                n4 = (n = this.floatCoords[--i]);
                n3 = (n2 = this.floatCoords[--i]);
                while (i > 0) {
                    float n5 = this.floatCoords[--i];
                    float n6 = this.floatCoords[--i];
                    if (n6 < n2) {
                        n2 = n6;
                    }
                    if (n5 < n) {
                        n = n5;
                    }
                    if (n6 > n3) {
                        n3 = n6;
                    }
                    if (n5 > n4) {
                        n4 = n5;
                    }
                    else {
                        continue;
                    }
                }
            }
            else {
                n = (n2 = (n3 = (n4 = 0.0f)));
            }
            return new Rectangle2D.Float(n2, n, n3 - n2, n4 - n);
        }
        
        @Override
        public final PathIterator getPathIterator(AffineTransform affineTransform) {
            if (affineTransform == null) {
                return new Path2D.Float.CopyIterator(this);
            }
            return new Path2D.Float.TxIterator(this, affineTransform);
        }
        
        @Override
        public final Object clone() {
            if ((this instanceof GeneralPath)) {
                return new GeneralPath(this);
            }
            return new Path2D.Float(this);
        }
        
        private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
            super.writeObject(objectOutputStream, false);
        }
        
        private void readObject(ObjectInputStream objectInputStream) throws ClassNotFoundException, IOException {
            super.readObject(objectInputStream, false);
        }
        
        static class CopyIterator extends Path2D.Iterator
        {
            float[] floatCoords;
            
            CopyIterator(Path2D.Float float1) {
                super(float1);
                this.floatCoords = float1.floatCoords;
            }
            
            @Override
            public int currentSegment(float[] array) {
                int n = this.path.pointTypes[this.typeIdx];
                int n2 = Path2D.Float.CopyIterator.curvecoords[n];
                if (n2 > 0) {
                    System.arraycopy(this.floatCoords, this.pointIdx, array, 0, n2);
                }
                return n;
            }
            
            @Override
            public int currentSegment(double[] array) {
                int n = this.path.pointTypes[this.typeIdx];
                int n2 = Path2D.Float.CopyIterator.curvecoords[n];
                if (n2 > 0) {
                    for (int i = 0; i < n2; ++i) {
                        array[i] = (double)this.floatCoords[this.pointIdx + i];
                    }
                }
                return n;
            }
        }
        
        static class TxIterator extends Path2D.Iterator
        {
            float[] floatCoords;
            AffineTransform affine;
            
            TxIterator(Path2D.Float float1, AffineTransform affine) {
                super(float1);
                this.floatCoords = float1.floatCoords;
                this.affine = affine;
            }
            
            @Override
            public int currentSegment(float[] array) {
                int n = this.path.pointTypes[this.typeIdx];
                int n2 = Path2D.Float.TxIterator.curvecoords[n];
                if (n2 > 0) {
                    this.affine.transform(this.floatCoords, this.pointIdx, array, 0, n2 / 2);
                }
                return n;
            }
            
            @Override
            public int currentSegment(double[] array) {
                int n = this.path.pointTypes[this.typeIdx];
                int n2 = Path2D.Float.TxIterator.curvecoords[n];
                if (n2 > 0) {
                    this.affine.transform(this.floatCoords, this.pointIdx, array, 0, n2 / 2);
                }
                return n;
            }
        }
    }
    
    public static class Double extends Path2D implements Serializable
    {
        transient double[] doubleCoords;
        private static final long serialVersionUID = 1826762518450014216L;
        
        public Double() {
            this(1, 20);
        }
        
        public Double(int n) {
            this(n, 20);
        }
        
        public Double(int n, int n2) {
            super(n, n2);
            this.doubleCoords = new double[n2 * 2];
        }
        
        public Double(Shape shape) {
            this(shape, null);
        }
        
        public Double(Shape shape, AffineTransform affineTransform) {
            if ((shape instanceof Path2D)) {
                Path2D path2D = (Path2D)shape;
                this.setWindingRule(path2D.windingRule);
                this.numTypes = path2D.numTypes;
                this.pointTypes = Arrays.copyOf(path2D.pointTypes, path2D.numTypes);
                this.numCoords = path2D.numCoords;
                this.doubleCoords = path2D.cloneCoordsDouble(affineTransform);
            }
            else {
                PathIterator pathIterator = shape.getPathIterator(affineTransform);
                this.setWindingRule(pathIterator.getWindingRule());
                this.pointTypes = new byte[20];
                this.doubleCoords = new double[40];
                this.append(pathIterator, false);
            }
        }
        
        @Override
        float[] cloneCoordsFloat(AffineTransform affineTransform) {
            float[] array = new float[this.numCoords];
            if (affineTransform == null) {
                for (int i = 0; i < this.numCoords; ++i) {
                    array[i] = (float)this.doubleCoords[i];
                }
                return array;
            }
            affineTransform.transform(this.doubleCoords, 0, array, 0, this.numCoords / 2);
            return array;
        }
        
        @Override
        double[] cloneCoordsDouble(AffineTransform affineTransform) {
            if (affineTransform == null) {
                return Arrays.copyOf(this.doubleCoords, this.numCoords);
            }
            double[] copy = new double[this.numCoords];
            affineTransform.transform(this.doubleCoords, 0, copy, 0, this.numCoords / 2);
            return copy;
        }
        
        @Override
        void append(float n, float n2) {
            this.doubleCoords[this.numCoords++] = (double)n;
            this.doubleCoords[this.numCoords++] = (double)n2;
        }
        
        @Override
        void append(double n, double n2) {
            this.doubleCoords[this.numCoords++] = n;
            this.doubleCoords[this.numCoords++] = n2;
        }
        
        @Override
        Point2D getPoint(int n) {
            return new Point2D.Double(this.doubleCoords[n], this.doubleCoords[n + 1]);
        }
        
        @Override
        void needRoom(boolean b, int n) {
            if (this.numTypes == 0 && b) {
                throw new IllegalPathStateException("missing initial moveto in path definition");
            }
            if (this.numTypes >= this.pointTypes.length) {
                this.pointTypes = Path2D.expandPointTypes(this.pointTypes, 1);
            }
            if (this.numCoords > this.doubleCoords.length - n) {
                this.doubleCoords = expandCoords(this.doubleCoords, n);
            }
            return;
        }
        
        static double[] expandCoords(double[] array, int n) {
            int n2 = array.length;
            int n3 = n2 + n;
            if (n3 < n2) {
                throw new ArrayIndexOutOfBoundsException("coords exceeds maximum capacity !");
            }
            if ((n2 = n2) > 1000) {
                n2 = Math.max(1000, n2 >> 3);
            }
            else if (n2 < 10) {
                n2 = 10;
            }
            assert n2 > n;
            int n4 = n2 + n2;
            if (n4 < n3) {
                n4 = Integer.MAX_VALUE;
            }
            try {
                return Arrays.copyOf(array, n4);
            }
            catch (OutOfMemoryError outOfMemoryError) {
                if (n4 == n3) {
                    throw outOfMemoryError;
                }
                n4 = n3 + (n4 - n3) / 2;
                return Arrays.copyOf(array, n4);
            }
        }
        
        @Override
        public final synchronized void moveTo(double n, double n2) {
            if (this.numTypes > 0 && this.pointTypes[this.numTypes - 1] == 0) {
                this.doubleCoords[this.numCoords - 2] = n;
                this.doubleCoords[this.numCoords - 1] = n2;
            }
            else {
                this.needRoom(false, 2);
                this.pointTypes[this.numTypes++] = 0;
                this.doubleCoords[this.numCoords++] = n;
                this.doubleCoords[this.numCoords++] = n2;
            }
        }
        
        @Override
        public final synchronized void lineTo(double n, double n2) {
            this.needRoom(true, 2);
            this.pointTypes[this.numTypes++] = 1;
            this.doubleCoords[this.numCoords++] = n;
            this.doubleCoords[this.numCoords++] = n2;
        }
        
        @Override
        public final synchronized void quadTo(double n, double n2, double n3, double n4) {
            this.needRoom(true, 4);
            this.pointTypes[this.numTypes++] = 2;
            this.doubleCoords[this.numCoords++] = n;
            this.doubleCoords[this.numCoords++] = n2;
            this.doubleCoords[this.numCoords++] = n3;
            this.doubleCoords[this.numCoords++] = n4;
        }
        
        @Override
        public final synchronized void curveTo(double n, double n2, double n3, double n4, double n5, double n6) {
            this.needRoom(true, 6);
            this.pointTypes[this.numTypes++] = 3;
            this.doubleCoords[this.numCoords++] = n;
            this.doubleCoords[this.numCoords++] = n2;
            this.doubleCoords[this.numCoords++] = n3;
            this.doubleCoords[this.numCoords++] = n4;
            this.doubleCoords[this.numCoords++] = n5;
            this.doubleCoords[this.numCoords++] = n6;
        }
        
        @Override
        int pointCrossings(double n, double n2) {
            if (this.numTypes == 0) {
                return 0;
            }
            double[] doubleCoords = this.doubleCoords;
            double x2;
            double x1 = x2 = doubleCoords[0];
            double y2;
            double y1 = y2 = doubleCoords[1];
            int n3 = 0;
            int n4 = 2;
            for (int i = 1; i < this.numTypes; ++i) {
                switch (this.pointTypes[i]) {
                    case 0: {
                        if (y2 != y1) {
                            n3 += Curve.pointCrossingsForLine(n, n2, x2, y2, x1, y1);
                        }
                        x2 = (x1 = doubleCoords[n4++]);
                        y2 = (y1 = doubleCoords[n4++]);
                        break;
                    }
                    case 1: {
                        int n5 = n3;
                        double x3 = x2;
                        double y3 = y2;
                        double x4 = doubleCoords[n4++];
                        double n6;
                        n3 = n5 + Curve.pointCrossingsForLine(n, n2, x3, y3, x4, n6 = doubleCoords[n4++]);
                        x2 = x4;
                        y2 = n6;
                        break;
                    }
                    case 2: {
                        int n7 = n3;
                        double x5 = x2;
                        double y4 = y2;
                        double xc = doubleCoords[n4++];
                        double yc = doubleCoords[n4++];
                        double x6 = doubleCoords[n4++];
                        double n8;
                        n3 = n7 + Curve.pointCrossingsForQuad(n, n2, x5, y4, xc, yc, x6, n8 = doubleCoords[n4++], 0);
                        x2 = x6;
                        y2 = n8;
                        break;
                    }
                    case 3: {
                        int n9 = n3;
                        double x7 = x2;
                        double y5 = y2;
                        double xc2 = doubleCoords[n4++];
                        double yc2 = doubleCoords[n4++];
                        double xc3 = doubleCoords[n4++];
                        double yc3 = doubleCoords[n4++];
                        double x8 = doubleCoords[n4++];
                        double n10;
                        n3 = n9 + Curve.pointCrossingsForCubic(n, n2, x7, y5, xc2, yc2, xc3, yc3, x8, n10 = doubleCoords[n4++], 0);
                        x2 = x8;
                        y2 = n10;
                        break;
                    }
                    case 4: {
                        if (y2 != y1) {
                            n3 += Curve.pointCrossingsForLine(n, n2, x2, y2, x1, y1);
                        }
                        x2 = x1;
                        y2 = y1;
                        break;
                    }
                }
            }
            if (y2 != y1) {
                n3 += Curve.pointCrossingsForLine(n, n2, x2, y2, x1, y1);
            }
            return n3;
        }
        
        @Override
        int rectCrossings(double n, double n2, double n3, double n4) {
            if (this.numTypes == 0) {
                return 0;
            }
            double[] doubleCoords = this.doubleCoords;
            double n5;
            double x1 = n5 = doubleCoords[0];
            double n6;
            double y1 = n6 = doubleCoords[1];
            int n7 = 0;
            int n8 = 2;
            for (int n9 = 1; n7 != Integer.MIN_VALUE && n9 < this.numTypes; ++n9) {
                switch (this.pointTypes[n9]) {
                    case 0: {
                        if (n5 != x1 || n6 != y1) {
                            n7 = Curve.rectCrossingsForLine(n7, n, n2, n3, n4, n5, n6, x1, y1);
                        }
                        n5 = (x1 = doubleCoords[n8++]);
                        n6 = (y1 = doubleCoords[n8++]);
                        break;
                    }
                    case 1: {
                        double x2 = doubleCoords[n8++];
                        double y2 = doubleCoords[n8++];
                        n7 = Curve.rectCrossingsForLine(n7, n, n2, n3, n4, n5, n6, x2, y2);
                        n5 = x2;
                        n6 = y2;
                        break;
                    }
                    case 2: {
                        int crossings = n7;
                        double x3 = n5;
                        double y3 = n6;
                        double xc = doubleCoords[n8++];
                        double yc = doubleCoords[n8++];
                        double x4 = doubleCoords[n8++];
                        double n10;
                        n7 = Curve.rectCrossingsForQuad(crossings, n, n2, n3, n4, x3, y3, xc, yc, x4, n10 = doubleCoords[n8++], 0);
                        n5 = x4;
                        n6 = n10;
                        break;
                    }
                    case 3: {
                        int crossings2 = n7;
                        double x5 = n5;
                        double y4 = n6;
                        double xc2 = doubleCoords[n8++];
                        double yc2 = doubleCoords[n8++];
                        double xc3 = doubleCoords[n8++];
                        double yc3 = doubleCoords[n8++];
                        double x6 = doubleCoords[n8++];
                        double n11;
                        n7 = Curve.rectCrossingsForCubic(crossings2, n, n2, n3, n4, x5, y4, xc2, yc2, xc3, yc3, x6, n11 = doubleCoords[n8++], 0);
                        n5 = x6;
                        n6 = n11;
                        break;
                    }
                    case 4: {
                        if (n5 != x1 || n6 != y1) {
                            n7 = Curve.rectCrossingsForLine(n7, n, n2, n3, n4, n5, n6, x1, y1);
                        }
                        n5 = x1;
                        n6 = y1;
                        break;
                    }
                }
            }
            if (n7 != Integer.MIN_VALUE && (n5 != x1 || n6 != y1)) {
                n7 = Curve.rectCrossingsForLine(n7, n, n2, n3, n4, n5, n6, x1, y1);
            }
            return n7;
        }
        
        @Override
        public final void append(PathIterator pathIterator, boolean b) {
            double[] array = new double[6];
            while (!pathIterator.isDone()) {
                switch (pathIterator.currentSegment(array)) {
                    case 0: {
                        if (!b || this.numTypes < 1 || this.numCoords < 1) {
                            this.moveTo(array[0], array[1]);
                            break;
                        }
                        if (this.pointTypes[this.numTypes - 1] != 4 && this.doubleCoords[this.numCoords - 2] == array[0] && this.doubleCoords[this.numCoords - 1] == array[1]) {
                            break;
                        }
                        this.lineTo(array[0], array[1]);
                        break;
                    }
                    case 1: {
                        this.lineTo(array[0], array[1]);
                        break;
                    }
                    case 2: {
                        this.quadTo(array[0], array[1], array[2], array[3]);
                        break;
                    }
                    case 3: {
                        this.curveTo(array[0], array[1], array[2], array[3], array[4], array[5]);
                        break;
                    }
                    case 4: {
                        this.closePath();
                        break;
                    }
                }
                pathIterator.next();
                b = false;
            }
        }
        
        @Override
        public final void transform(AffineTransform affineTransform) {
            affineTransform.transform(this.doubleCoords, 0, this.doubleCoords, 0, this.numCoords / 2);
        }
        
        @Override
        public final synchronized Rectangle2D getBounds2D() {
            int i = this.numCoords;
            double n4;
            double n3;
            double n2;
            double n;
            if (i > 0) {
                n4 = (n = this.doubleCoords[--i]);
                n3 = (n2 = this.doubleCoords[--i]);
                while (i > 0) {
                    double n5 = this.doubleCoords[--i];
                    double n6 = this.doubleCoords[--i];
                    if (n6 < n2) {
                        n2 = n6;
                    }
                    if (n5 < n) {
                        n = n5;
                    }
                    if (n6 > n3) {
                        n3 = n6;
                    }
                    if (n5 > n4) {
                        n4 = n5;
                    }
                    else {
                        continue;
                    }
                }
            }
            else {
                n = (n2 = (n3 = (n4 = 0.0)));
            }
            return new Rectangle2D.Double(n2, n, n3 - n2, n4 - n);
        }
        
        @Override
        public final PathIterator getPathIterator(AffineTransform affineTransform) {
            if (affineTransform == null) {
                return new Path2D.Double.CopyIterator(this);
            }
            return new Path2D.Double.TxIterator(this, affineTransform);
        }
        
        @Override
        public final Object clone() {
            return new Path2D.Double(this);
        }
        
        private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
            super.writeObject(objectOutputStream, true);
        }
        
        private void readObject(ObjectInputStream objectInputStream) throws ClassNotFoundException, IOException {
            super.readObject(objectInputStream, true);
        }
        
        static class CopyIterator extends Path2D.Iterator
        {
            double[] doubleCoords;
            
            CopyIterator(Path2D.Double double1) {
                super(double1);
                this.doubleCoords = double1.doubleCoords;
            }
            
            @Override
            public int currentSegment(float[] array) {
                int n = this.path.pointTypes[this.typeIdx];
                int n2 = Path2D.Double.CopyIterator.curvecoords[n];
                if (n2 > 0) {
                    for (int i = 0; i < n2; ++i) {
                        array[i] = (float)this.doubleCoords[this.pointIdx + i];
                    }
                }
                return n;
            }
            
            @Override
            public int currentSegment(double[] array) {
                int n = this.path.pointTypes[this.typeIdx];
                int n2 = Path2D.Double.CopyIterator.curvecoords[n];
                if (n2 > 0) {
                    System.arraycopy(this.doubleCoords, this.pointIdx, array, 0, n2);
                }
                return n;
            }
        }
        
        static class TxIterator extends Path2D.Iterator
        {
            double[] doubleCoords;
            AffineTransform affine;
            
            TxIterator(Path2D.Double double1, AffineTransform affine) {
                super(double1);
                this.doubleCoords = double1.doubleCoords;
                this.affine = affine;
            }
            
            @Override
            public int currentSegment(float[] array) {
                int n = this.path.pointTypes[this.typeIdx];
                int n2 = Path2D.Double.TxIterator.curvecoords[n];
                if (n2 > 0) {
                    this.affine.transform(this.doubleCoords, this.pointIdx, array, 0, n2 / 2);
                }
                return n;
            }
            
            @Override
            public int currentSegment(double[] array) {
                int n = this.path.pointTypes[this.typeIdx];
                int n2 = Path2D.Double.TxIterator.curvecoords[n];
                if (n2 > 0) {
                    this.affine.transform(this.doubleCoords, this.pointIdx, array, 0, n2 / 2);
                }
                return n;
            }
        }
    }
    
    abstract static class Iterator implements PathIterator
    {
        int typeIdx;
        int pointIdx;
        Path2D path;
        static final int[] curvecoords;
        
        Iterator(Path2D path) {
            this.path = path;
        }
        
        @Override
        public int getWindingRule() {
            return this.path.getWindingRule();
        }
        
        @Override
        public boolean isDone() {
            return this.typeIdx >= this.path.numTypes;
        }
        
        @Override
        public void next() {
            this.pointIdx += Path2D.Iterator.curvecoords[this.path.pointTypes[this.typeIdx++]];
        }
        
        static {
            curvecoords = new int[] { 2, 2, 4, 6, 0 };
        }
    }
}
